<?php
/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0
 * @package    CartBounty Pro - Save and recover abandoned carts for WooCommerce
 * @subpackage CartBounty Pro - Save and recover abandoned carts for WooCommerce/includes
 * @author     Streamline.lv
 */
 
class CartBounty_Pro_Activator{

	/**
	 * Creating table in the database
	 *
	 * @since    1.3
	 */
	public static function activate() {
		
		//Deactivating CartBounty plugin
		require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		deactivate_plugins('woo-save-abandoned-carts/cartbounty-abandoned-carts.php');
		
		/**
		* Creating table
		*/
		global $wpdb;
		
		$cart_table = $wpdb->prefix . CARTBOUNTY_PRO_TABLE_NAME;
		$old_cart_table = $wpdb->prefix . "captured_wc_fields_pro";
		$charset_collate = $wpdb->get_charset_collate();

		$sql = "CREATE TABLE $cart_table (
			id BIGINT(20) NOT NULL AUTO_INCREMENT,
			name VARCHAR(60),
			surname VARCHAR(60),
			email VARCHAR(100),
			phone VARCHAR(20),
			phone_consent TINYINT DEFAULT 0,
			location VARCHAR(100),
			cart_contents LONGTEXT,
			cart_total DECIMAL(10,2),
			currency VARCHAR(10),
			time DATETIME DEFAULT '0000-00-00 00:00:00',
			session_id VARCHAR(60),
			language VARCHAR(25),
			other_fields LONGTEXT,
			mail_sent TINYINT NOT NULL DEFAULT 0,
			mc_synced TINYINT NOT NULL DEFAULT 0,
			mc_coupon_synced DATETIME DEFAULT '0000-00-00 00:00:00',
			mc_excluded TINYINT DEFAULT 0,
			last_synced DATETIME DEFAULT '0000-00-00 00:00:00',
			ac_last_synced DATETIME DEFAULT '0000-00-00 00:00:00',
			ac_contact_id INT DEFAULT 0,
			ac_customer_id INT DEFAULT 0,
			ac_cart_id INT DEFAULT 0,
			ac_excluded TINYINT DEFAULT 0,
			gr_last_synced DATETIME DEFAULT '0000-00-00 00:00:00',
			gr_contact_id VARCHAR(60),
			gr_cart_id VARCHAR(60),
			gr_excluded TINYINT DEFAULT 0,
			wp_unsubscribed TINYINT DEFAULT 0,
			wp_steps_completed INT(3) DEFAULT 0,
			wp_last_sent DATETIME DEFAULT '0000-00-00 00:00:00',
			wp_complete TINYINT DEFAULT 0,
			wp_excluded TINYINT DEFAULT 0,
			sms_unsubscribed TINYINT DEFAULT 0,
			sms_steps_completed INT(3) DEFAULT 0,
			sms_last_sent DATETIME DEFAULT '0000-00-00 00:00:00',
			sms_complete TINYINT DEFAULT 0,
			sms_excluded TINYINT DEFAULT 0,
			wh_last_synced DATETIME DEFAULT '0000-00-00 00:00:00',
			wh_excluded TINYINT DEFAULT 0,
			pn_subscription LONGTEXT,
			pn_steps_completed INT(3) DEFAULT 0,
			pn_last_sent DATETIME DEFAULT '0000-00-00 00:00:00',
			pn_complete TINYINT DEFAULT 0,
			pn_excluded TINYINT DEFAULT 0,
			coupons LONGTEXT,
			refferals LONGTEXT,
			paused TINYINT DEFAULT 0,
			type VARCHAR(10) DEFAULT 0,
			wc_order_id BIGINT(20) DEFAULT 0,
			saved_via VARCHAR(10),
			PRIMARY KEY (id)
		) $charset_collate;";

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta( $sql );
		
		/**
		* Resets table Auto increment index to 1
		*/
		$sql = "ALTER TABLE $cart_table AUTO_INCREMENT = 1";
		dbDelta( $sql );

		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$reports = new CartBounty_Pro_Reports();

		//Automatically import carts from the Free version on plugin activation / update
		$admin->import_carts();

		/**
		 * Handling cart transfer from the old captured_wc_fields_pro table to new one
		 * Temporary block since version 8.1.1. Will be removed in future versions
		 *
		 * @since    8.1.1
		 */
		function cartbounty_pro_transfer_carts( $wpdb, $cart_table, $old_cart_table ){
		    if(!cartbounty_pro_old_table_exists( $wpdb, $old_cart_table )){ //If old table no longer exists, exit
		    	return;
		    }
		    if(!get_option('cartbounty_pro_transferred_table')){ //If we have not yet transfered carts to the new table
		    	$old_carts = $wpdb->get_results( //Selecting all rows that are not empty
	    			"SELECT * FROM $old_cart_table
	    			WHERE cart_contents != ''
	    			"
		    	);

		    	if($old_carts){ //If we have carts
		    		$imported_cart_count = 0;
		    		$batch_count = 0; //Keeps count of current batch of data to insert
		    		$batches = array(); //Array containing the batches of import since SQL is having troubles importing too many rows at once
					$abandoned_cart_data = array();
					$placeholders = array();

					foreach($old_carts as $key => $cart){ // Looping through abandoned carts to create the arrays
						$batch_count++;

						array_push(
							$abandoned_cart_data,
							sanitize_text_field( $cart->id ),
							sanitize_text_field( $cart->name ),
							sanitize_text_field( $cart->surname ),
							sanitize_email( $cart->email ),
							sanitize_text_field( $cart->phone ),
							sanitize_text_field( $cart->location ),
							sanitize_text_field( $cart->cart_contents ),
							sanitize_text_field( $cart->cart_total ),
							sanitize_text_field( $cart->currency ),
							sanitize_text_field( $cart->time ),
							sanitize_text_field( $cart->session_id ),
							sanitize_text_field( $cart->mail_sent ),
							sanitize_text_field( $cart->mc_synced ),
							sanitize_text_field( $cart->other_fields ),
							sanitize_text_field( $cart->last_synced ),
							sanitize_text_field( $cart->ac_last_synced ),
							sanitize_text_field( $cart->ac_customer_id ),
							sanitize_text_field( $cart->ac_cart_id ),
							sanitize_text_field( $cart->gr_last_synced ),
							sanitize_text_field( $cart->gr_contact_id ),
							sanitize_text_field( $cart->gr_cart_id ),
							sanitize_text_field( $cart->type )
						);
						$placeholders[] = "( %d, %s, %s, %s, %s, %s, %s, %0.2f, %s, %s, %s, %d, %d, %s, %s, %s, %d, %d, %s, %s, %s, %s)";

						if($batch_count >= 100){ //If we get a full batch, add it to the array and start preparing a new one
							$batches[] = array(
								'data'			=>	$abandoned_cart_data,
								'placeholders'	=>	$placeholders
							);
							$batch_count = 0;
							$abandoned_cart_data = array();
							$placeholders = array();
						}
					}

					//In case something is left at the end of the loop, we add it to the batches so we do not loose any abandoned carts during the import process
					if($abandoned_cart_data){
						$batches[] = array(
							'data'			=>	$abandoned_cart_data,
							'placeholders'	=>	$placeholders
						);
					}
					
					foreach ($batches as $key => $batch) { //Looping through the batches and importing the carts
						$query = "INSERT INTO ". $cart_table ." (id, name, surname, email, phone, location, cart_contents, cart_total, currency, time, session_id, mail_sent, mc_synced, other_fields, last_synced, ac_last_synced, ac_customer_id, ac_cart_id, gr_last_synced, gr_contact_id, gr_cart_id, type) VALUES ";
						$query .= implode(', ', $batch['placeholders']);
						$count = $wpdb->query( $wpdb->prepare("$query ", $batch['data']));
						$imported_cart_count = $imported_cart_count + $count;
					}
		    	}

		    	update_option('cartbounty_pro_transferred_table', true); //Making sure the user is not allowed to transfer carts more than once
		    	$wpdb->query( "DROP TABLE IF EXISTS $old_cart_table" ); //Removing old table from the database
		    }
		}

		/**
		 * Determine if we have old CartBounty cart table still present
		 * Temporary block since version 8.1.1. Will be removed in future versions
		 *
		 * @since    8.1.1
		 * @return 	 Boolean
		 */
		function cartbounty_pro_old_table_exists( $wpdb, $old_cart_table ){
			$exists = false;
			$table_exists = $wpdb->query(
				"SHOW TABLES LIKE '{$old_cart_table}'"
			);
			if($table_exists){ //In case table exists
				$exists = true;
			}
			return $exists;
		}

		//Temporary function since version 8.1.1. Will be removed in future releases
		cartbounty_pro_transfer_carts( $wpdb, $cart_table, $old_cart_table );

		/**
		 * Since version 8.1
		 * This code will be removed in later versions
		 */
		if (get_option( 'cartbounty_pro_captured_abandoned_cart_count' )){
			update_option( 'cartbounty_pro_recoverable_cart_count', get_option( 'cartbounty_pro_captured_abandoned_cart_count' ));
			delete_option( 'cartbounty_pro_captured_abandoned_cart_count' );
		}
		/* End of this temporary block */

		/**
		 * Since version 9.2.1
		 * This code will be removed in later versions
		 */
		if(get_option('cartbounty_pro_automation_setup')){
			update_option('cartbounty_pro_mc_setup', get_option('cartbounty_pro_automation_setup'));
			delete_option('cartbounty_pro_automation_setup');
		}
		if(get_option('cartbounty_pro_activecampaign_automation_setup')){
			update_option('cartbounty_pro_ac_setup', get_option('cartbounty_pro_activecampaign_automation_setup'));
			delete_option('cartbounty_pro_activecampaign_automation_setup');
		}
		if(get_option('cartbounty_pro_getresponse_automation_setup')){
			update_option('cartbounty_pro_gr_setup', get_option('cartbounty_pro_getresponse_automation_setup'));
			delete_option('cartbounty_pro_getresponse_automation_setup');
		}
		/* End of this temporary block */

		//Setting default Exit Intent type if it has not been previously set
		add_option('cartbounty_pro_exit_intent_type', 1);

		//Setting default Early capture type if it has not been previously set
		add_option('cartbounty_pro_early_capture_type', 1);

		//Setting default Exit Intent field type to capture in case it has not been set before
		add_option('cartbounty_pro_exit_intent_field_type', 'email');

		//Setting default Early capture field type to capture in case it has not been set before
		add_option('cartbounty_pro_early_capture_field_type', 'email');

		//Making sure that automated coupon deletion is enabled by default
		add_option('cartbounty_pro_delete_coupons', 1);

		//Setting default quick stats that are enabled by default
		add_option( 'cartbounty_pro_active_quick_stats', $reports->get_default_reports() );

		//Setting default charts that are enabled by default
		add_option( 'cartbounty_pro_active_charts', $reports->get_default_reports( 'charts' ) );

		/**
		 * Since version 9.4.3
		 * This code will be removed in later versions
		 */
		if(get_option('cartbounty_pro_automation_sent_emails')){
			update_option('cartbounty_pro_automation_sends', get_option('cartbounty_pro_automation_sent_emails'));
			delete_option('cartbounty_pro_automation_sent_emails');
		}

		if ( get_option( 'cartbounty_pro_wpml_strings_registered' ) ){ //In order to automatically register new strings if we update CartBounty
			update_option( 'cartbounty_pro_wpml_strings_registered', false );
		}

		/**
		 * Since version 9.9.3
		 * Transfering time to miliseconds
		 * This code will be removed in later versions
		 */
		function transfer_time_to_miliseconds(){

			if( CARTBOUNTY_PRO_VERSION_NUMBER == get_option( 'cartbounty_pro_version_number' ) || empty( get_option( 'cartbounty_pro_version_number' ) ) ){ //If this is a fresh install or plugin activation
				update_option( 'cartbounty_pro_converted_minutes_to_miliseconds', true ); //setting this variable as we do not require to convert minutes to miliseconds for new installs or activations
				return;
			}

			if( get_option( 'cartbounty_pro_converted_minutes_to_miliseconds' ) ) return;

			$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
			$wordpress_steps = get_option( 'cartbounty_pro_automation_steps' );
			$bulkgate_steps = get_option( 'cartbounty_pro_bulkgate_steps' );
			$mailchimp_coupon = get_option( 'cartbounty_pro_mailchimp_coupon' );
			$getresponse_coupon = get_option( 'cartbounty_pro_getresponse_coupon' );
			$activecampaign_coupon = get_option( 'cartbounty_pro_activecampaign_coupon' );
			$webhook_coupon = get_option( 'cartbounty_pro_webhook_coupon' );
			$notification_frequency = get_option( 'cartbounty_pro_notification_frequency' );
			$tab_notification_interval = get_option( 'cartbounty_pro_tab_notification_interval' );

			//Converting WordPres recovery time intervals
			if( $wordpress_steps ){

				if( is_array( $wordpress_steps ) && !empty( $wordpress_steps ) ){
					foreach( $wordpress_steps as $key => $step ){
						
						if( isset( $wordpress_steps[$key]['interval'] ) ){
							$wordpress_steps[$key]['interval'] = $admin->convert_minutes_to_miliseconds( $step['interval'] );
						}

						if( isset( $wordpress_steps[$key]['coupon_expiry'] ) ){
							$wordpress_steps[$key]['coupon_expiry'] = $admin->convert_minutes_to_miliseconds( $step['coupon_expiry'] );
						}
					}
					update_option( 'cartbounty_pro_automation_steps', $wordpress_steps );
				}
			}

			//Converting BulkGate recovery time intervals
			if( $bulkgate_steps ){

				if( is_array( $bulkgate_steps ) && !empty( $bulkgate_steps ) ){
					foreach( $bulkgate_steps as $key => $step ) {

						if( isset( $bulkgate_steps[$key]['interval'] ) ){
							$bulkgate_steps[$key]['interval'] = $admin->convert_minutes_to_miliseconds( $step['interval'] );
						}

						if( isset( $bulkgate_steps[$key]['coupon_expiry'] ) ){
							$bulkgate_steps[$key]['coupon_expiry'] = $admin->convert_minutes_to_miliseconds( $step['coupon_expiry'] );
						}
					}
					update_option( 'cartbounty_pro_bulkgate_steps', $bulkgate_steps );
				}
			}

			//Converting coupon expiry data
			$recovery_coupon_data = array(
				'cartbounty_pro_mailchimp_coupon'		=>	$mailchimp_coupon,
				'cartbounty_pro_getresponse_coupon'		=>	$getresponse_coupon,
				'cartbounty_pro_activecampaign_coupon'	=>	$activecampaign_coupon,
				'cartbounty_pro_webhook_coupon'			=>	$webhook_coupon
			);

			foreach( $recovery_coupon_data as $key => $coupon ) {

				if( !empty( $coupon ) ){

					if( isset( $coupon['coupon_expiry'] ) ){
						$coupon['coupon_expiry'] = $admin->convert_minutes_to_miliseconds( $coupon['coupon_expiry'] );
						update_option( $key, $coupon );
					}
				}
			}

			//Converting Notification time interval
			if( !empty( $notification_frequency ) ){

				if( isset( $notification_frequency['hours'] ) ){
					$notification_frequency['interval'] = $admin->convert_minutes_to_miliseconds( $notification_frequency['hours'] );
					update_option( 'cartbounty_pro_notification_frequency', $notification_frequency );
				}
			}

			//Converting Tab notification time interval
			if( !empty( $tab_notification_interval ) ){

				if( isset( $tab_notification_interval['miliseconds'] ) ){
					$tab_notification_interval['interval'] = $tab_notification_interval['miliseconds'];
					update_option( 'cartbounty_pro_tab_notification_interval', $tab_notification_interval );
				}
			}

			update_option( 'cartbounty_pro_converted_minutes_to_miliseconds', true );

			/**
			 * Since version 9.9.3
			 * Due to moving to a different time interval add_custom_wp_cron_intervals() functions
			 */
			if( wp_next_scheduled( 'cartbounty_pro_remove_empty_carts_hook' ) ){
				wp_clear_scheduled_hook( 'cartbounty_pro_remove_empty_carts_hook' );
			}

			if( wp_next_scheduled( 'cartbounty_pro_delete_coupons_hook' ) ){
				wp_clear_scheduled_hook( 'cartbounty_pro_delete_coupons_hook' );
			}

		}

		transfer_time_to_miliseconds();

		/**
		 * Since version 9.11
		 * This code will be removed in later versions
		 */
		if(get_option('cartbounty_pro_ghost_cart_count')){
			update_option('cartbounty_pro_anonymous_cart_count', get_option('cartbounty_pro_ghost_cart_count'));
			delete_option('cartbounty_pro_ghost_cart_count');
		}

		if(get_option('cartbounty_pro_exclude_ghost_carts')){
			update_option('cartbounty_pro_exclude_anonymous_carts', get_option('cartbounty_pro_exclude_ghost_carts'));
			delete_option('cartbounty_pro_exclude_ghost_carts');
		}

		if(get_option('cartbounty_pro_delete_ghost_carts')){
			update_option('cartbounty_pro_delete_anonymous_carts', get_option('cartbounty_pro_delete_ghost_carts'));
			delete_option('cartbounty_pro_delete_ghost_carts');
		}

		/**
		 * Since version 10.0
		 * This code will be removed in later versions
		 */
		if( get_option( 'cartbounty_pro_review_submitted' ) ){
			update_option( 'cartbounty_pro_submitted_notices', array( 'review' => 1 ) );
			delete_option( 'cartbounty_pro_review_submitted' );
		}
		/* End of this temporary block */
	}
}