<?php
/**
 * Fired during plugin deactivation.
 *
 * This class defines all code necessary to run during the plugin's deactivation.
 *
 * @since      1.0
 * @package    CartBounty Pro - Save and recover abandoned carts for WooCommerce
 * @subpackage CartBounty Pro - Save and recover abandoned carts for WooCommerce/includes
 * @author     Streamline.lv
 */
class CartBounty_Pro_Deactivator{

	/**
	 * Deactivation function
	 *
	 * @since    1.1
	 */
	public static function deactivate() {
		$admin = new CartBounty_Pro_Admin( CARTBOUNTY_PRO_PLUGIN_NAME_SLUG, CARTBOUNTY_PRO_VERSION_NUMBER );
		$hooks = array(
			'cartbounty_pro_sync_hook',
			'cartbounty_pro_notification_sendout_hook',
			'cartbounty_pro_remove_empty_carts_hook',
			'cartbounty_pro_delete_anonymous_carts_hook',
			'cartbounty_pro_delete_coupons_hook',
			'cartbounty_pro_clear_update_hook' //No longer used since version 6.8. Will be removed in later 
		);

		foreach( $hooks as $key => $hook ){

			if( $admin->action_scheduler_enabled() ){ //If WooCommerce Action scheduler library exists
				as_unschedule_action( $hook, array(), CARTBOUNTY_PRO ); //Deactivating scheduled Action Scheduler actions

			}else{ //Fallback to WP Cron and clear these events
				wp_clear_scheduled_hook( $hook ); //Deactivating scheduled WP Cron actions
			}
		}

		delete_option( 'cartbounty_pro_update_last_checked' );
		delete_option( 'cartbounty_pro_update_link' );

		delete_transient( 'cartbounty_pro_mailchimp_api_key' );
		delete_transient( 'cartbounty_pro_recoverable_cart_count' );
		delete_transient( 'cartbounty_pro_mailchimp_list_id' );
		delete_transient( 'cartbounty_pro_activecampaign_key' );
		delete_transient( 'cartbounty_pro_activecampaign_url' );
		delete_transient( 'cartbounty_pro_activecampaign_external_store_id' );
		delete_transient( 'cartbounty_free_table_exists' );
		delete_transient( 'cartbounty_pro_request_sent' );
		delete_transient( 'cartbounty_pro_getresponse_key' );
	}
}